-- E52103Y.ADA

-- CHECK WHETHER A NULL ARRAY WITH ONE DIMENSION OF LENGTH GREATER THAN
-- INTEGER'LAST RAISES NUMERIC_ERROR, CONSTRAINT_ERROR OR NO EXCEPTION,
-- EITHER WHEN DECLARED OR  ASSIGNED.

-- CHECK THAT LENGTHS MUST MATCH IN ARRAY AND SLICE ASSIGNMENTS.
--    MORE SPECIFICALLY, TEST THAT ARRAY ASSIGNMENTS WITH MATCHING
--    LENGTHS DO NOT CAUSE  CONSTRAINT_ERROR  TO BE RAISED AND
--    ARE PERFORMED CORRECTLY.
--    (OVERLAPS BETWEEN THE OPERANDS OF THE ASSIGNMENT STATEMENT
--    ARE TREATED ELSEWHERE.)


-- THIS IS A SPECIAL CASE IN

--    DIVISION  D :  NULL ARRAYS WHOSE LENGTHS ARE NOT DETERMINABLE
--                   STATICALLY

-- WHICH (THE SPECIAL CASE) TREATS TWO-DIMENSIONAL ARRAYS WHOSE LENGTH
--    ALONG ONE DIMENSION IS GREATER THAN  INTEGER'LAST  AND WHOSE
--    LENGTH ALONG THE OTHER DIMENSION IS  0 .


-- RM  07/31/81
-- SPS 03/22/83
-- JBG 05/02/83
-- JBG 06/01/85
-- EG  10/28/85  FIX NUMERIC_ERROR/CONSTRAINT_ERROR ACCORDING TO
--               AI-00387.
-- LDC 06/01/88  CHANGED HEADER COMMENT TO INDICATE CONSTRAINT_ERROR
--               IS ALLOWED.  ADDED CODE TO PREVENT DEAD VARIABLE
--               OPTIMIZATION.

WITH REPORT;
PROCEDURE  E52103Y  IS

     USE  REPORT ;

BEGIN

     TEST( "E52103Y","CHECK WHETHER NUMERIC_ERROR/CONSTRAINT_ERROR " &
                     "OR NO EXCEPTION IS RAISED WHEN DIMENSION OF " &
                     "AN ARRAY HAS LENGTH > INTEGER'LAST");
     BEGIN

          DECLARE

               TYPE  TA42  IS  ARRAY(
                    INTEGER RANGE IDENT_INT( 13 )..IDENT_INT( 12 ),
                    INTEGER RANGE IDENT_INT(-2)..IDENT_INT(INTEGER'LAST)
                                    )  OF BOOLEAN ;

               SUBTYPE  TA41  IS  TA42 ;

               ARR41  :  TA41 ;
               ARR42  :  TA42 ;

          BEGIN

               COMMENT ("NO EXCEPTION FOR ARRAY DECLARATION");

               -- NULL ARRAY ASSIGNMENT:

               ARR42 := ARR41 ;
               IF ARR42'LENGTH(1) /= 0 THEN
                    FOR I IN TA42'RANGE(2) LOOP
                         ARR41(13,I) := IDENT_BOOL(ARR42(13,I));
                    END LOOP;
               END IF;

               COMMENT ("NO EXCEPTION RAISED FOR NULL ARRAY " &
                        "ASSIGNMENT");

          EXCEPTION

               WHEN  NUMERIC_ERROR =>
                    COMMENT ("NUMERIC_ERROR RAISED IN LENGTH " &
                             "COMPARISON");

               WHEN CONSTRAINT_ERROR =>
                    COMMENT ("CONSTRAINT_ERROR RAISED IN LENGTH " &
                             "COMPARISON");

               WHEN  OTHERS  =>
                    FAILED( "OTHER EXCEPTION RAISED  -  SUBTEST 2" );

          END ;

     EXCEPTION

          WHEN NUMERIC_ERROR =>
               COMMENT ("NUMERIC_ERROR RAISED BY DECLARATION OF " &
                        "NULL ARRAY TYPE WITH ONE DIMENSION > " &
                        "INTEGER'LAST");

          WHEN CONSTRAINT_ERROR =>
               COMMENT ("CONSTRAINT_ERROR RAISED BY DECLARATION OF " &
                        "NULL ARRAY TYPE WITH ONE DIMENSION > " &
                        "INTEGER'LAST");

          WHEN OTHERS =>
               FAILED ("SOME OTHER EXCEPTION RAISED");

     END;

     -------------------------------------------------------------------


     RESULT ;


END E52103Y;
